

#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"


//---------------------------------------------------------------------------------------------------

#define UART0_Rx_Complete   		USART_RXC_vect	//  _VECTOR(11)  /* USART Rx Complete */
#define UART0_Data_Register_Empty	USART_UDRE_vect //   /* USART, Data Register Empty */
#define UART0_UCSRA   	UCSRA
#define UART0_UCSRB  	UCSRB
#define UART0_DATA     	UDR
#define UART0_UBRR  	UBRR
#define UART0_UCSRC  	UCSRC

#define UART0_UDRIE    	UDRIE//      UART.  
#define UART0_RXCIE    	RXCIE//    . 
#define UART0_TXCIE		TXCIE//    .



#define UART_BAUD_SELECT ((xtalCpu)/((baudRate)*16l)-1)


#define UART_Tx_BUF_MASK	(16-1)	//0b111


volatile uint8_t UART_TxHead, UART_TxTail;
uint8_t	UART_Tx_buf[UART_Tx_BUF_MASK+1];
UARTSTR UartTxStr;



//---------------------------------------------------------------
void UART_init(void)
{
	UBRRL = 51;//UART_BAUD_SELECT;// Set baud rate 
	UCSRB = (0<<RXEN)|(1<<TXEN)|(0<<RXCIE);// Enable receiver //and transmitter 
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);// Set frame format: 8data, 2stop bit 
	//USBS-2stop bit //0b11<<UCSZ0 8 bit
}


//---------------------------------------------------------------
static void uart_putc(uint8_t data)
{
	uint8_t tmphead;

	tmphead  = (UART_TxHead + 1) & UART_Tx_BUF_MASK;
    
	while ( tmphead == UART_TxTail ){
		;// wait for free space in buffer
	}
	UART_Tx_buf[tmphead] = data;
	UART_TxHead = tmphead;
	// enable UDRE interrupt 
	UART0_UCSRB   |= _BV(UART0_UDRIE);
}


//---------------------------------------------------------------
void uart_puts_str(void){
	
	uint8_t crc=0;
	uint8_t i, *p=(uint8_t *) &UartTxStr;
	for(i=0; i<sizeof(UartTxStr); i++, p++){
		if ( i < (sizeof(UartTxStr)-1) )//   crc
			{
			crc^=*p;
			}
			else
			*p=crc;
		
		uart_putc(*p);
		}
}


//---------------------------------------------------------------
ISR(UART0_Data_Register_Empty)
{
	if ( UART_TxHead != UART_TxTail) {
		uint8_t head_tx;
		head_tx = (UART_TxTail + 1) & UART_Tx_BUF_MASK;
        UART0_DATA = UART_Tx_buf[head_tx];  // start transmission 
		UART_TxTail = head_tx;
    }else{
        // tx buffer empty, disable UDRE interrupt 
        UART0_UCSRB &= ~_BV(UART0_UDRIE);
    }
}












//ISR(__vector_default)
//{
// user code here
//}
//USART_TXC_vect //USART, Tx Complete

/*

#define USART_TXC_vect			_VECTOR(13) // USART, Tx Complete 
#define USART_UDRE_vect			_VECTOR(12) // USART Data Register Empty 
#define USART_RXC_vect			_VECTOR(11) // USART, Rx Complete

----UCSRA----
RXC -   .    1    
     (  UDR).  
    .   RXCIE (RXCIErt)  
 UCSRB (UCSR/jB) ,      
     

  -   .    1     
       ,    
 UDR     .   TXCIE () 
 UCSRB (UCSR/?B) ,      
   .      
     ,  
  . 1 

UDRE -    .     1 
    (      
UDR    ).   , 
       .   UDRIE 
(UDRIErt)  UCSRB (UCSR/iB) ,   
    .   , 
     

FE -   .    1   
 , . .   -    
0.     -,  1 

DOR -  .    1,     
 -       
 ,     (  ).   
         
   

UPE -    .    1,   , 
   ,    . 
         0 

U2X -   .      1,   
       16  
8,         
 .         
       

MPCM -   .      1,  
   ,  . , 
   ,  


*/
